# -*- coding: utf-8 -*-
# moebel.py

from grafikfenster import *
from math import radians

from abc import ABC,abstractmethod # abstrakte Klasse realisieren

### ---------------------------------------------------
class Moebel(ABC):
    """Klasse Moebel
    Oberklasse für das Zeichnen und Bearbeiten von
    Möbel-Symbolen fuer den Raumplaner
    Sie implementiert die Komponente für ein Moebel-Kompositum
    mit leeren Methoden für die Blatt-Moebel-Objekte."""
    
    def __init__(self,
                 xPos,
                 yPos,
                 breite,
                 tiefe,
                 winkel,
                 farbe,
                 sichtbar):
        """Konstruktor mit vordefinierten Parametern
        ist bei Moebel nicht sinnvoll"""
        self.__x=xPos
        self.__y=yPos
        self.__b=breite
        self.__t=tiefe
        self.__w=winkel
        self.__f=farbe
        self.__s=sichtbar
        self.__gruppenDrehung=(0,(0,0))
        if sichtbar: self.Zeige()
        
    def FuegeHinzu(self, moebel):
        '''fügt für eine MoebelGruppe eine neue Moebel-Komponente hinzu'''
        pass
        
    def Entferne(self, moebel):
        '''entfernt bei einer MoebelGruppe die Moebel-Komponente'''
        pass
        
    def GibKindobjekt(self, position):
        """gibt für eine MoebelGruppe das Kindobjekt zur Position zurück"""
        return None

    def _aktualisiereDaten(self,x,y,b,t):
        '''wird nach Aenderung der Gruppe aufgerufen'''
        self.__x=x
        self.__y=y
        self.__b=b
        self.__t=t
    
    @abstractmethod
    def GibFigur(self):
        """Definiert die zu zeichnende Figur."""
        return NotImplemented

    def GibZeichenPfad(self):
        return Zeichenflaeche.GibZeichenflaeche().GibGC().CreatePath()

    def GibTransformation(self):
        """erzeugt die zugehörige Transformation"""
        gc = Zeichenflaeche.GibZeichenflaeche().GibGC()
        matrix=gc.CreateMatrix()
        #self.__gruppenDrehung verarbeiten
        if self.__gruppenDrehung[0]!=0:
            zX,zY=self.__gruppenDrehung[1]
            matrix.Translate(zX, zY)
            matrix.Rotate(radians(self.__gruppenDrehung[0]))
            matrix.Translate(-zX, -zY)
        matrix.Translate(self.__x+round(self.__b/2),
                         self.__y+round(self.__t/2))
        matrix.Rotate(radians(self.__w))
        matrix.Translate(-round(self.__b/2), -round(self.__t/2))
        return matrix

    def Transformiere(self, path):
        """Transformiert den übergebenen Pfad"""
        path.Transform(self.GibTransformation())
        return path

    def GibXPosition(self):
        """Get-Methode fuer die xPosition"""
        return self.__x
    
    def GibYPosition(self):
        """Get-Methode fuer die yPosition"""
        return self.__y
    
    def GibBreite(self):
        """Get-Methode fuer die Breite"""
        return self.__b
    
    def GibTiefe(self):
        """Get-Methode fuer die Tiefe"""
        return self.__t
    
    def GibOrientierung(self):
        """Get-Methode fuer die Orientierung"""
        return self.__w
    
    def GibFarbe(self):
        """Get-Methode fuer die Farbe"""
        return self.__f
    
    def GibSichtbar(self):
        """Get-Methode fuer die Sichtbarkeit"""
        return self.__s

    def BewegeHorizontal(self, weite):
        """Veraendernde Methode fuer die x-Position"""
        self.Loesche()
        self.__x += weite
        self.Zeichne()
    
    def BewegeVertikal(self, weite):
        """Veraendernde Methode fuer die y-Position"""
        self.Loesche()
        self.__y += weite
        self.Zeichne()
    
    def Drehe(self, winkel, zentrum=None):
        """Veraendernde Methode fuer die Orientierung [Winkel]"""
        self.Loesche()
        if zentrum==None:
            self.__w += winkel
        else: # Drehung um uebergebenes Zentrum, eigener Winkel bleibt
            self.__gruppenDrehung=(winkel, zentrum)
        self.Zeichne()

    def Verberge(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert False"""
        self.__s = False
        self.Loesche()

    def Zeige(self):
        """Veraendernde Methode fuer die Sichtbarkeit mit Wert True"""
        self.__s = True
        self.Zeichne()

    def Loesche(self):
        """Entfernt ein Objekt aus der Darstellung."""
        Zeichenflaeche.GibZeichenflaeche().Entferne(self)

    def Zeichne(self):
        """Fuegt das Objekt den darzustellenden hinzu."""
        Zeichenflaeche.GibZeichenflaeche().Zeichne(self)

### ---------------------------------------------------
class MoebelApp(wx.App):
    """Test-Anwendung speziell fuer Stuhl"""
    def OnInit(self):
        self.fenster = GrafikFenster(None, "Raumplaner-Grafik")
        self.SetTopWindow(self.fenster)
        self.fenster.Show(True)
        self.fenster.panel.Refresh()
        self.fenster.ZeigeShellFrame()
        #self.fenster.ZeigeFillingFrame()
        self.TestAnwendung()
        return True

    def TestAnwendung(self):
        """Allein fuer die Testanwendung:
        sollte nicht gehen!"""
        global moebel
        try:
            moebel=Moebel(70, 30, 120, 60, 0, 'red', True)
        except Exception as e:
            print( str(e) )
        
### ---------------------------------------------------
if __name__ == '__main__':
    app = MoebelApp(redirect=False)
    # Parameterwert True, wenn Ausgaben in der Standard E/A angezeigt werden sollen
    app.MainLoop()
